/*
 * DO NOT EDIT THIS FILE - it is generated by Glade.
 */

GtkWidget* create_messagebox_dialog (void);
GtkWidget* create_new_rss_dialog (void);
GtkWidget* create_new_board_dialog (void);
GtkWidget* create_new_pop_dialog (void);
GtkWidget* create_colorselection_dialog (void);
GtkWidget* create_global_pinnipede_options_dialog (void);
GtkWidget* create_edit_dialog (void);
GtkWidget* create_bronson_wizard (void);
GtkWidget* create_sitelist_dialog (void);
GtkWidget* create_pinnipede_site_colors_dialog (void);
GtkWidget* create_change_board_settings_dialog (void);
GtkWidget* create_change_rss_settings_dialog (void);
GtkWidget* create_change_pop_settings_dialog (void);
GtkWidget* create_grab_cookie_dialog (void);
GtkWidget* create_fontselectiondialog (void);
GtkWidget* create_proxy_dialog (void);
