#include <gtk/gtk.h>


void
on_message_tags_are__htmlentitised_1_activate
                                        (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_message_tags_are_raw1_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_the_backend_has_no_pants1_activate  (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
new_board_dialog_OK                    (GtkButton       *button,
                                        gpointer         user_data);

void
on_mark_id_gaps_checkbutton_toggled    (GtkToggleButton *togglebutton,
                                        gpointer         user_data);

void
on_mark_id_gaps_checkbutton_toggled    (GtkToggleButton *togglebutton,
                                        gpointer         user_data);

void
edit_bt_clicked                        (GtkButton       *button,
                                        gpointer         user_data);

void
on_bottom1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data);

void
on_left1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data);
